from packaging import version
from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from netbox.plugins import PluginTemplateExtension
from .template_content_functions import create_text, create_url, config_for_modul, create_QRCode

# ******************************************************************************************
# Contains the main functionalities of the plugin and thus creates the content for the 
# individual modules, e.g: Device, Rack etc.
# ******************************************************************************************

##################################
# Class for creating the plugin content
class QRCode(PluginTemplateExtension):

    ##################################          
    # Creates a plug-in view for a label.
    # --------------------------------
    # Parameter:
    #   labelDesignNo: Which label design should be loaded.
    def Create_SubPluginContent(self, labelDesignNo):
        
        thisSelf = self

        obj = self.context['object'] # An object of the type Device, Rack etc.

        # Config suitable for the module
        config = config_for_modul(thisSelf, labelDesignNo)

        # Abort if no config data. 
        if config is None: 
            return '' 

        # Get URL for QR code
        url = create_url(thisSelf, config, obj)

        # Create a QR code
        qrCode = create_QRCode(url, config)

        # Create the text for the label if required.
        text = create_text(config, obj, qrCode)

        # Create plugin using template
        try:
            if version.parse(settings.RELEASE.version).major >= 3:

                render = self.render(
                    'netbox_qrcode/qrcode3.html', extra_context={
                                                                    'title': config.get('title'),
                                                                    'labelDesignNo': labelDesignNo,
                                                                    'qrCode': qrCode, 
                                                                    'with_text': config.get('with_text'),
                                                                    'text': text,
                                                                    'text_location': config.get('text_location'),
                                                                    'text_align_horizontal': config.get('text_align_horizontal'),
                                                                    'text_align_vertical': config.get('text_align_vertical'),
                                                                    'font': config.get('font'),
                                                                    'font_size': config.get('font_size'),
                                                                    'font_weight': config.get('font_weight'),
                                                                    'font_color': config.get('font_color'),
                                                                    'with_qr': config.get('with_qr'),
                                                                    'label_qr_width': config.get('label_qr_width'),
                                                                    'label_qr_height': config.get('label_qr_height'),
                                                                    'label_qr_text_distance': config.get('label_qr_text_distance'),
                                                                    'label_width': config.get('label_width'),
                                                                    'label_height': config.get('label_height'), 
                                                                    'label_edge_top': config.get('label_edge_top'),
                                                                    'label_edge_left': config.get('label_edge_left'),
                                                                    'label_edge_right': config.get('label_edge_right'),
                                                                    'label_edge_bottom': config.get('label_edge_bottom')
                                                                }

                )
            
                return render
            else:
                # Versions 1 and 2 are no longer supported.
                return self.render(
                    'netbox_qrcode/qrcode.html', extra_context={'image': qrCode}
                )
        except ObjectDoesNotExist:
            return ''

    ##################################
    # Create plugin content
    # - First, a plugin view is created for the first label.
    # - If there are further configuration entries for the object/model (e.g. device, rack etc.),
    #   further label views are also created as additional plugin views.
    def Create_PluginContent(self):

        # First Plugin Content
        pluginContent = QRCode.Create_SubPluginContent(self, 1) 

        # Check whether there is another configuration for the object, e.g. device, rack, etc.
        # Support up to 10 additional label configurations (objectName_2 to ..._10) per object (e.g. device, rack, etc.).

        config = self.context['config'] # Django configuration

        for i in range(2, 11):

            configName = self.models[0].replace('dcim.', '') + '_' + str(i)
            obj_cfg = config.get(configName) # Load configuration for additional label if possible.

            if(obj_cfg):
                pluginContent += QRCode.Create_SubPluginContent(self, i) # Add another plugin view
            else:
                break
        
        return pluginContent

##################################
# The following section serves to integrate the plugin into Netbox Core.
        
# Class for creating a QR code for the model: Device
class DeviceQRCode(QRCode):
    models = ('dcim.device',)

    def right_page(self):
        return self.Create_PluginContent()

# Class for creating a QR code for the model: Rack
class RackQRCode(QRCode):
    models = ('dcim.rack',)

    def right_page(self):
        return self.Create_PluginContent()

# Class for creating a QR code for the model: Cable
class CableQRCode(QRCode):
    models = ('dcim.cable',)

    def left_page(self):
        return self.Create_PluginContent()

# Class for creating a QR code for the model: Location
class LocationQRCode(QRCode):
    models = ('dcim.location',)

    def left_page(self):
        return self.Create_PluginContent()

# Class for creating a QR code for the model: Power Feed
class PowerFeedQRCode(QRCode):
    models = ('dcim.powerfeed',)

    def right_page(self):
        return self.Create_PluginContent()

# Class for creating a QR code for the model: Power Panel
class PowerPanelQRCode(QRCode):
    models = ('dcim.powerpanel',)

    def right_page(self):
        return self.Create_PluginContent()

# Class for dcim.module
class ModuleQRCode(QRCode):
    models = ('dcim.module',)

    def right_page(self):
        return self.Create_PluginContent()

##################################
# Other plugins support

# Commenting out (for now) - make this work on core models first.
# Class for creating a QR code for the Plugin: Netbox-Inventory (https://github.com/ArnesSI/netbox-inventory)
#class Plugin_Netbox_Inventory(QRCode):
#    models = ()'netbox_inventory.asset' # Info for Netbox in which model the plugin should be integrated.
#
#    def right_page(self):
#        return self.Create_PluginContent()

# Connects Netbox Core with the plug-in classes
# Removed , Plugin_Netbox_Inventory]
template_extensions = [DeviceQRCode, ModuleQRCode, RackQRCode, CableQRCode, LocationQRCode, PowerFeedQRCode, PowerPanelQRCode]


from .utilities import get_img_b64, get_qr
from django.template import engines

# ******************************************************************************************
# For better clarity, the sub-functions of template_content.py have been outsourced.
# ******************************************************************************************

##################################
# The configuration is taken and all fields that are module-specific (e.g. Device, Rack, etc.) are replaced.
# --------------------------------
# Parameter:
#   labelDesignNo: Which label design should be loaded.
#   parentSelf: Self from Parrent Function
def config_for_modul(parentSelf, labelDesignNo):

    # Copy so that the Runtime data is not changed.
    config = parentSelf.context['config'].copy() # From Netbox Config File

    # Create suffix to read the correct module configuration.
    confModulsufix = str() # None if the first standard label

    if(labelDesignNo >= 2):
            confModulsufix = '_' + str(labelDesignNo)

    # Collect the QR code plugin configuration for the specific object such as device, rack etc.
    # and overwrite the default configuration fields.
    obj_cfg = config.get(parentSelf.models[0].replace('dcim.', '') + confModulsufix) # get spezific object settings
    
    if obj_cfg is not None: 
        config.update(obj_cfg) # Ovverride default confiv Values
        return config
    else:
        return config # No module customisation

##################################
# Create QR-Code
# --------------------------------
# Parameter:
#   text: Text for QR-Code
#   config: From the Netbox configuration file
def create_QRCode(text, config):

    # Collect the configuration entries that begin with "qr_.
    # These are required to generate the QR code.
    qr_args = {}
    for k, v in config.items():
        if k.startswith('qr_'):
            qr_args[k.replace('qr_', '')] = v

    # Create a QR code
    qrCode = get_qr(text, **qr_args)
    return get_img_b64(qrCode)


##################################
# Create URL for QR code
# --------------------------------
# Parameter:
#   config: From the Netbox configuration file
#   obj: Data from the model (e.g. device, rack, etc.)
#   request: HTML Request Information
def create_url(parentSelf, config, obj):

    request = parentSelf.context['request'] # HTML Request Informations

    if config.get('url_template'):
        # A user-defined design specification of the URL is provided in ninja2 format.
        django_engine = engines["django"]
        template = django_engine.from_string(config.get('url_template')) # Custom template for URL design.
        return template.render({'obj': obj}) # Replace placeholder
    else:
        return request.build_absolute_uri(obj.get_absolute_url()) # URL to the requested page

##################################
# Create text for label
# --------------------------------
# Parameter:
#   config: From the Netbox configuration file
#   obj: Data from the model (e.g. device, rack, etc.)
#   qrCode: QR-Code Image 
def create_text(config, obj, qrCode):

    text = str()

    if config.get('with_text'):
        if config.get('text_template'):
            return get_text_template(config, obj, qrCode) # Create text content based on the Ninja2 template from the user
        else:
            return get_text_fields(config, obj) # Use the list of variables from the Config.

##################################
# A user-defined design specification of the text is provided in ninja2 format.
# --------------------------------
# Parameter:
#   config: From the Netbox configuration file
#   obj: Data from the model (e.g. device, rack, etc.)
#   qrCode: QR-Code Image (To create a freely defined label with QR code.)
def get_text_template(config, obj, qrCode):

    django_engine = engines["django"]
    template = django_engine.from_string(config.get('text_template')) # Get Custom Template
    logo = config.get('logo')
    return template.render({'obj': obj, 
                            'logo': logo,
                            'qrCode': qrCode}) # Replace placeholder

##################################
# Retrieves all values from the object (e.g. device, rack, etc.)
# depending on the configuration parameter that are to be displayed in list form and prepares them.
# --------------------------------
# Parameter:
#   config: From the Netbox configuration file
#   obj: Data from the model (e.g. device, rack, etc.)
def get_text_fields(config, obj):

    text = []

    for text_field in config.get('text_fields', []):
        cfn = None
        if '.' in text_field:
            try:
                text_field, cfn = text_field.split('.')
            except ValueError:
                cfn = None
        if getattr(obj, text_field, None):
            if cfn:
                try:
                    if getattr(obj, text_field).get(cfn):
                        text.append('{}'.format(getattr(obj, text_field).get(cfn)))
                except AttributeError:
                    # fix for nb3.3: trying to get cable termination and device in same way as custom field
                    if type(getattr(obj, text_field)) is list:
                        first_element = next(iter(getattr(obj, text_field)), None)
                        if first_element and getattr(first_element, cfn, None):
                            text.append('{}'.format(getattr(first_element, cfn)))
            else:
                text.append('{}'.format(getattr(obj, text_field)))

    # Append user-defined text to the end.
    custom_text = config.get('custom_text')

    if custom_text:
        text.append(custom_text)

    # Convert text list to string with line breaks.
    return '<br>'.join(text)